package com.bcxin.tenant.open.dubbo.reader.providers.impls;

import com.bcxin.tenant.open.document.domains.documents.NotifyMessageDocument;
import com.bcxin.tenant.open.document.domains.documents.NotifyMessageDocument$;
import com.bcxin.tenant.open.document.domains.repositories.NotifyMessageDocumentRepository;
import com.bcxin.tenant.open.infrastructures.components.JsonProvider;
import com.bcxin.tenant.open.infrastructures.utils.ExceptionUtil;
import com.bcxin.tenant.open.jdks.NotifyMessageRpcProvider;
import com.bcxin.tenant.open.jdks.responses.NotifyMessageResponse;
import com.redis.om.spring.search.stream.EntityStream;
import com.redis.om.spring.search.stream.SearchStream;
import org.apache.dubbo.config.annotation.DubboService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.Collection;
import java.util.Collections;
import java.util.Map;
import java.util.stream.Collectors;

@DubboService
public class NotifyMessageRpcProviderImpl implements NotifyMessageRpcProvider {
    private static final Logger logger = LoggerFactory.getLogger(NotifyMessageRpcProviderImpl.class);

    private final EntityStream entityStream;
    private final NotifyMessageDocumentRepository notifyMessageDocumentRepository;
    private final JsonProvider jsonProvider;

    public NotifyMessageRpcProviderImpl(EntityStream entityStream,
                                        NotifyMessageDocumentRepository notifyMessageDocumentRepository, JsonProvider jsonProvider) {
        this.entityStream = entityStream;
        this.notifyMessageDocumentRepository = notifyMessageDocumentRepository;
        this.jsonProvider = jsonProvider;
    }

    @Override
    public Collection<NotifyMessageResponse> getInstantMessages(Collection<String> employeeIds) {
        if (CollectionUtils.isEmpty(employeeIds)) {
            return Collections.EMPTY_LIST;
        }

        employeeIds = employeeIds.stream()
                .filter(ii-> StringUtils.hasLength(ii))
                .collect(Collectors.toList());
        SearchStream<NotifyMessageDocument> notifyMessageDocumentSearchStream =
                this.entityStream.of(NotifyMessageDocument.class);

        notifyMessageDocumentSearchStream = notifyMessageDocumentSearchStream.filter(NotifyMessageDocument$.EMPLOYEE_ID.in(
                employeeIds.toArray(ix -> new String[ix])
        ));

        Collection<NotifyMessageDocument> documents = notifyMessageDocumentSearchStream.collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(documents)) {
            try {
                this.notifyMessageDocumentRepository.deleteAll(documents);
            } catch (Exception ex) {
                logger.error("移除当前用户({})的信息:大小={}",
                        employeeIds.stream().collect(Collectors.joining(",")),
                        documents.size(), ex
                );
            }
        }

        Collection<NotifyMessageResponse> data =
                documents
                        .stream().map(ii -> {
                            Map content = jsonProvider.toObject(Map.class, ii.getContent());
                            return NotifyMessageResponse.create(ii.getId(), ii.getMessageType(), content);
                        })
                        .collect(Collectors.toList());

        return data;
    }

    @Override
    public void doConfirm(String referenceNumber, Collection<String> employeeIds) {
        try {
            SearchStream<NotifyMessageDocument> notifyMessageDocumentSearchStream =
                    this.entityStream.of(NotifyMessageDocument.class);

            notifyMessageDocumentSearchStream = notifyMessageDocumentSearchStream
                    .filter(NotifyMessageDocument$.EMPLOYEE_ID.in(employeeIds.toArray(new String[employeeIds.size()])));

            /**
             * 很奇怪的问题, 支持in,但是使用eq是不行的.
             */
            notifyMessageDocumentSearchStream = notifyMessageDocumentSearchStream
                    .filter(NotifyMessageDocument$.REFERENCE_NUMBER.eq(referenceNumber));

            Collection<NotifyMessageDocument> documents =
                    notifyMessageDocumentSearchStream.collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(documents)) {
                this.notifyMessageDocumentRepository.deleteAll(documents);
            }
        } catch (Exception ex) {
            logger.error("确认用户(roomId={};employeeId={})已经查看信息完毕:{}", referenceNumber, employeeIds.stream().collect(Collectors.joining(",")), ExceptionUtil.getStackMessage(ex));
        }
    }
}
