package com.bcxin.tenant.open.dubbo.reader.providers.transfers;

import com.bcxin.tenant.open.document.domains.documents.*;
import com.bcxin.tenant.open.infrastructures.TenantContext;
import com.bcxin.tenant.open.infrastructures.TenantEmployeeContext;
import com.bcxin.tenant.open.infrastructures.constants.BusinessConstants;
import com.bcxin.tenant.open.infrastructures.enums.DeskType;
import com.bcxin.tenant.open.infrastructures.utils.BusinessUtil;
import com.bcxin.tenant.open.infrastructures.utils.StationCodeUtils;
import com.bcxin.tenant.open.jdks.requests.EmployeeSearchRequest;
import com.bcxin.tenant.open.jdks.requests.SecurityStationReaderRequest;
import com.bcxin.tenant.open.jdks.responses.*;
import com.redis.om.spring.search.stream.EntityStream;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.stream.Collectors;

@Component
public class DataTransferImpl implements DataTransfer {

    @Override
    public EmployeeDetailResponse translate(DeskType deskType, RdEmployeeDocument document, EntityStream entityStream) {
        if (document == null) {
            return null;
        }

        boolean follow = false;
        if (document.getFollowedDeviceNos() != null) {
            TenantEmployeeContext.TenantUserModel userModel = TenantContext.getInstance().getUserContext().get();
            if (userModel != null) {
                follow = document.getFollowedDeviceNos().stream().anyMatch(ii ->
                        StringUtils.hasLength(ii) && ii.equalsIgnoreCase(userModel.getTencentUserId())
                );
            }
        }

        String projectId = null;
        String projectName = null;
        if (deskType == DeskType.TemporarySecurity && !BusinessConstants.DefaultEmptyValue.equals(document.getSecurityStationId())) {
            RdSecurityStationDocument securityStationDocument =
                    entityStream.of(RdSecurityStationDocument.class).filter(RdSecurityStationDocument$.ID.eq(document.getSecurityStationId()))
                            .findFirst().orElse(null);
            if (securityStationDocument != null) {
                projectId = securityStationDocument.getProjectId();
                projectName = securityStationDocument.getProjectName();
            }
        }

        String stationType = "";
        if (document.getStationTypes() != null) {
            stationType = document.getStationTypes().stream().filter(ii -> StringUtils.hasLength(ii)).findFirst().orElse(null);
        }
        EmployeeDetailResponse response =
                EmployeeDetailResponse.create(
                        document.getId(),
                        document.getName(),
                        document.getLonLat() == null ? null : document.getLonLat().getY(),
                        document.getLonLat() == null ? null : document.getLonLat().getX(),
                        StringUtils.hasLength(document.getIdCardNo()) ? document.getIdCardNo() : "",
                        StringUtils.hasLength(document.getContact()) ? document.getContact() : "",
                        document.getOrganizationId(),
                        document.getCompanyName(),
                        document.getAuthenticatedStatus().getTypeName(),
                        document.getDutySignInEnumType(),
                        document.getDispatchable(),
                        document.getLastDutyStatusChangedTime(),
                        document.getMilitaryStatus(),
                        document.getSecurityCertificateNo(),
                        document.getSecurityStationId(),
                        document.getSecurityStationName(),
                        /**
                         * 如果用户无绑定设备, 那么使用自己的原Im账号进行调度
                         */
                        StringUtils.hasLength(document.getDeviceNumber()) ? document.getDeviceNumber() : document.getTenantImUserId(),
                        document.getTenantImUserId(),
                        document.getSuperviseDepartId(),
                        document.getCId(), StringUtils.hasLength(document.getDeviceNumber()),
                        document.getSuperviseDepartName(),
                        document.getFollowedDeviceNos(),
                        document.getThirdParty(),
                        follow,
                        document.getSecurityStationRailIds(),
                        document.getSelectedResponsibleOfStationId(document.getSecurityStationId()),
                        document.getResourceTypes(),
                        document.getInstitutional(),
                        document.getScopePermissions(),
                        document.getProprietorCompanyId(),
                        document.getProprietorCompanyName(),
                        document.getOccupationType(),
                        document.getScopePermissions(),
                        document.getLastUpdatedTime(),
                        String.format("sync.%s", document.getLastDutyStatusChangedTime()),
                        document.isDomainAdmin(),
                        document.getResponsibleOfStationIds(),
                        document.getSuperviseRegionCode(),
                        document.getCLevel(),
                        projectId,
                        projectName,
                        stationType,
                        document.getCountOfStation()
                );

        return response;
    }

    @Override
    public Collection<EmployeeOverviewResponse> translate(Collection<DeskType> deskTypes,EmployeeSearchRequest.SearchModel model,
                                                          Collection<RdEmployeeDocument> documents,
                                                          EntityStream entityStream) {
        if (CollectionUtils.isEmpty(documents)) {
            return Collections.EMPTY_LIST;
        }


        Collection<RdSecurityStationDocument> securityStationDocuments = new ArrayList<>();
        if (model== EmployeeSearchRequest.SearchModel.Keyword && !CollectionUtils.isEmpty(deskTypes) && deskTypes.contains(DeskType.TemporarySecurity)  ) {
            Collection<String> stationIds =
                    documents.stream().map(ii -> ii.getSecurityStationId()).filter(ii -> !BusinessConstants.DefaultEmptyValue.equalsIgnoreCase(ii))
                            .distinct().collect(Collectors.toList());

            if (!CollectionUtils.isEmpty(stationIds)) {
                securityStationDocuments =
                        entityStream.of(RdSecurityStationDocument.class)
                                .filter(RdSecurityStationDocument$.ID.in(stationIds.toArray(ix -> new String[ix])))
                                .collect(Collectors.toList());
            }
        }

        Collection<RdSecurityStationDocument> finalSecurityStationDocuments = securityStationDocuments;
        return documents.stream().map(document -> {
            boolean follow = false;
            if (model != EmployeeSearchRequest.SearchModel.SecurityResource) {
                if (document.getFollowedDeviceNos() != null) {
                    TenantEmployeeContext.TenantUserModel userModel = TenantContext.getInstance().getUserContext().get();

                    if (userModel != null) {
                        follow = document.getFollowedDeviceNos().stream().anyMatch(ii ->
                                StringUtils.hasLength(ii) && ii.equalsIgnoreCase(userModel.getTencentUserId())
                        );
                    }
                }
            }

            String projectId = null;
            String projectName = null;
            RdSecurityStationDocument selectedStationDocument = finalSecurityStationDocuments.stream()
                    .filter(ii->ii.getId().equalsIgnoreCase(document.getSecurityStationId())).findFirst().orElse(null);
            if(selectedStationDocument!=null) {
                projectId = selectedStationDocument.getProjectId();
                projectName = selectedStationDocument.getProjectName();
            }

            String stationType = "";
            if (document.getStationTypes() != null) {
                stationType = document.getStationTypes().stream().filter(ii -> StringUtils.hasLength(ii)).findFirst().orElse(null);
            }
            return EmployeeOverviewResponse.create(
                    model,
                    document.getId(),
                    document.getOrganizationId(),
                    document.getName(),
                    StringUtils.hasLength(document.getIdCardNo()) ? document.getIdCardNo() : "",
                    StringUtils.hasLength(document.getContact()) ? document.getContact() : "",
                    document.getSecurityStationName(),
                    document.getCompanyName(),
                    document.getLonLat() == null ? null : document.getLonLat().getY(),
                    document.getLonLat() == null ? null : document.getLonLat().getX(),
                    document.getDutySignInEnumType(),
                    document.getDispatchable(),
                    /**
                     * 如果用户无绑定设备, 那么使用自己的原Im账号进行调度
                     */
                    StringUtils.hasLength(document.getDeviceNumber()) ? document.getDeviceNumber() : document.getTenantImUserId(),
                    document.getTenantImUserId(),
                    document.getSuperviseDepartId(),
                    document.getSecurityStationId(),
                    StringUtils.hasLength(document.getDeviceNumber()),
                    document.getCId(),
                    document.getFollowedDeviceNos(),
                    follow,
                    document.getSelectedResponsibleOfStationId(document.getSecurityStationId()),
                    document.getResourceTypes(),
                    document.getProprietorCompanyId(),
                    document.getProprietorCompanyName(),
                    projectId,
                    projectName,
                    stationType
            );

        }).collect(Collectors.toList());
    }

    @Override
    public CompanyDetailResponse translate(
            RdCompanyDocument companyDocument
            ,Integer countOfExamSiteRooms) {
        if (companyDocument == null) {
            return null;
        }
        boolean follow = false;
        if (companyDocument.getFollowedDeviceNos() != null) {
            TenantEmployeeContext.TenantUserModel userModel =
                    TenantContext.getInstance().getUserContext().get();
            if (userModel != null) {
                follow = companyDocument.getFollowedDeviceNos().stream().anyMatch(ii ->
                        StringUtils.hasLength(ii) && ii.equalsIgnoreCase(userModel.getTencentUserId())
                );
            }
        }

        return CompanyDetailResponse.create(
                companyDocument.getId(),
                companyDocument.getName(),
                companyDocument.getLegal(),
                companyDocument.getLegalTelephone(),
                companyDocument.getRegisterAddress(),
                companyDocument.getCountOfActivityStation(),
                companyDocument.getCountOfSecurityMan(),
                companyDocument.getLonLat() == null ? null : companyDocument.getLonLat().getY(),
                companyDocument.getLonLat() == null ? null : companyDocument.getLonLat().getX(),
                companyDocument.getInstitutional(),
                companyDocument.getOrganizationId(),
                countOfExamSiteRooms,
                companyDocument.getSuperviseDepartName(),
                companyDocument.getFollowedDeviceNos(),
                follow,
                companyDocument.getCountOfMan(),
                companyDocument.getResourceTypes(),
                companyDocument.getScopePermissions(),
                companyDocument.getLastUpdatedTime()
        );
    }

    @Override
    public SecurityStationDetailResponse translate(EntityStream entityStream,RdSecurityStationDocument document) {
        if (document == null) {
            return null;
        }

        boolean follow = false;
        if (document.getFollowedDeviceNos() != null) {
            TenantEmployeeContext.TenantUserModel userModel = TenantContext.getInstance().getUserContext().get();

            follow = document.getFollowedDeviceNos().stream().anyMatch(ii ->
                    StringUtils.hasLength(ii) && ii.equalsIgnoreCase(userModel.getTencentUserId())
            );
        }
        return SecurityStationDetailResponse.create(
                document.getId(),
                document.getName(),
                document.getKeyword(),
                document.getSiteState(),
                StationCodeUtils.getStationNames(document.getStationTypes()),
                document.getOrganizationId(),
                document.getCompanyName(),
                document.getPrincipalId(),
                document.getPrincipalName(),
                document.getPrincipalContact(),
                document.getServiceScope(),
                document.getAddress(),
                document.getLonLat() == null ? null : document.getLonLat().getY(),
                document.getLonLat() == null ? null : document.getLonLat().getX(),
                document.getCountOfSecurityMan(),
                document.getSuperviseDepartId(), document.getSuperviseDepartName(),
                document.getFollowedDeviceNos(),
                follow,
                document.getResourceTypes(),
                document.getInstitutional(),
                document.getCountOfCommunityUser(),
                document.getScopePermissions(),
                document.getProprietorCompanyId(),
                document.getProprietorCompanyName(),
                document.getScopePermissions(),
                document.getLastUpdatedTime(),
                document.getPerformRange(),
                document.getProjectId(),
                document.getProjectName()
        );
    }

    @Override
    public Collection<SecurityStationOverviewReaderResponse> translate2Stations(
            SecurityStationReaderRequest.SearchModel mode,
            Collection<RdSecurityStationDocument> documents) {
        if (CollectionUtils.isEmpty(documents)) {
            return Collections.emptyList();
        }

        return documents.stream().map(ix -> {
            boolean follow = false;
            if (ix.getFollowedDeviceNos() != null) {
                TenantEmployeeContext.TenantUserModel userModel = TenantContext.getInstance().getUserContext().get();

                follow = ix.getFollowedDeviceNos().stream().anyMatch(ii ->
                        StringUtils.hasLength(ii) && ii.equalsIgnoreCase(userModel.getTencentUserId())
                );
            }
            
            return
                    SecurityStationOverviewReaderResponse.create(
                            mode,
                            ix.getId(), ix.getName(),
                            ix.getOrganizationId(),
                            ix.getCompanyName(),
                            ix.getLonLat() == null ? null : ix.getLonLat().getY(),
                            ix.getLonLat() == null ? null : ix.getLonLat().getX(),
                            ix.getAddress(),
                            ix.getStationTypes(),
                            ix.getSuperviseDepartId(),
                            ix.getFollowedDeviceNos(),
                            follow,
                            ix.getPrincipalName(),
                            ix.getCountOfSecurityMan(),
                            ix.getResourceTypes(),
                            ix.getProjectId(),
                            ix.getProjectName()
                    );
        }).collect(Collectors.toList());
    }

    @Override
    public Collection<DeviceResponse> translate(Collection<RdDeviceDocument> documents, boolean isSecurityResourceRequest) {
        return documents.stream().map(ii -> {
            return translate(ii, isSecurityResourceRequest);
        }).collect(Collectors.toSet());
    }

    @Override
    public DeviceResponse translate(RdDeviceDocument ii, boolean isSecurityResourceRequest) {
        Double latitude = null, longitude = null;
        if (ii.getLonLat() != null) {
            latitude = ii.getLonLat().getY();
            longitude = ii.getLonLat().getX();
        }
        if (isSecurityResourceRequest) {
            return DeviceResponse.createForSecurityResource(ii.getId(), latitude, longitude, ii.getDeviceStatus());
        } else {
            return DeviceResponse.create(ii.getId(),
                    ii.getDeviceUid(),
                    ii.getDispatchNo(), ii.getDeviceModel(), ii.getOrganizationId(), ii.getOrganizationName(), ii.getSecurityStationId(),
                    ii.getSecurityStationName(), ii.getDeviceType(), latitude, longitude,
                    ii.getDeviceStatus(),ii.getLastSyncTime());
        }
    }
}
